//
//  CategoryProductsRequest.m
//  Compare Prices
//
//  Created by Jesper Forsberg on 2011-08-15.
//  Copyright 2011 Jesper Forsberg. All rights reserved.
//

#import "CategoryProductsRequest.h"
#import "Product.h"
#import "ProductStore.h"
#import "PriceHint.h"
#import "Category.h"
#import "JSONKit.h"

@interface CategoryProductsRequest ()
- (Category *)categoryFromJSON:json;
@end

@implementation CategoryProductsRequest

@synthesize page = page;
@synthesize minPrice = minPrice;
@synthesize maxPrice = maxPrice;
@synthesize sort = sort;
@synthesize searchQuery = searchQuery;

- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
        page = 1;
        pageSize = 10;
        minPrice = -1.f;
        maxPrice = -1.f;
    }
    
    return self;
}

- (void)dealloc {
    [self setSearchQuery:nil];
    [self setSort:nil];
    [super dealloc];
}

- (void)setCategoryId:(int)v
{
    categoryId = v;
}

- (void)setSort:(NSString *)v
{
    if (v != sort) {
        [sort release];
        sort = [v copy];
    }
}

- (void)setPageSize:(int)v
{
    pageSize = v;
}

- (void)setShowStores:(BOOL)v
{
    showStores = v;
}

- (Category *)categoryFromJSON:(id)json
{
    Category *category = [[Category alloc] init];
    
    category.id = [[json valueForKey:@"id"] intValue];
    category.parentId = [[json valueForKey:@"parentid"] intValue];
    category.name = [json valueForKey:@"name"];
    
    return [category autorelease];
}

- (CategoryProductsResponse *)query:(QuenchClient *)client
{
    if (!categoryId) {
        return nil;
    }
        
    NSString *url = [NSString stringWithFormat:@"/category/%d/products", categoryId];
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   [[NSNumber numberWithInt:self.page] stringValue], @"pg", 
                                   [[NSNumber numberWithInt:pageSize] stringValue], @"pgsize", nil];
        
    if (searchQuery) {
        [params setObject:searchQuery forKey:@"q"];
    }
    
    if (sort) {
        [params setObject:sort forKey:@"s"];
    }
    
    if (minPrice != -1.f) {
        [params setObject:[[NSNumber numberWithFloat:minPrice] stringValue] forKey:@"lp"];
    }
    
    if (maxPrice != -1.f) {
        [params setObject:[[NSNumber numberWithFloat:maxPrice] stringValue] forKey:@"up"];
    }
    
    if (showStores) {
        [params setObject:@"1" forKey:@"showstores"];
    }
    
    id jsonResponse = [[client queryResource:url withParams:[NSDictionary dictionaryWithDictionary:params]] objectFromJSONData];
    
    if (![jsonResponse isKindOfClass:[NSDictionary class]]) {
        return nil;
    }
    
    CategoryProductsResponse *response = [[CategoryProductsResponse alloc] init];
    Category *category = nil;
    if ([[jsonResponse allKeys] containsObject:@"category"]) {
        category = [self categoryFromJSON:[jsonResponse valueForKey:@"category"]];
        [response setCategory:category];
    }
    
    int hitCount = [[jsonResponse valueForKey:@"hitcount"] intValue];
    [response setHitCount:hitCount];
    
    int pageCount = [[jsonResponse valueForKey:@"pagecount"] intValue];
    [response setPageCount:pageCount];
    
    NSMutableArray *products = [NSMutableArray array];
    
    for (id productObj in [jsonResponse valueForKey:@"products"]) {
        Product *product = [[Product alloc] init];
        product.id = [[productObj valueForKey:@"id"] intValue];
        product.categoryId = [[productObj valueForKey:@"categoryid"] intValue];
        product.name = [productObj valueForKey:@"name"];
        product.info = [productObj valueForKey:@"description"];
        product.minPrice = [[productObj valueForKey:@"minprice"] floatValue];
        product.maxPrice = [[productObj valueForKey:@"maxprice"] floatValue];
        product.minPrice_c = [productObj valueForKey:@"minprice_c"];
        product.maxPrice_c = [productObj valueForKey:@"maxprice_c"];
        product.storeCount = [[productObj valueForKey:@"storecount"] intValue];
        product.imageURL = [productObj valueForKey:@"imageurl"];
        
        if (showStores) {
            NSMutableArray *stores = [NSMutableArray array];
            for (id storeObj in [productObj valueForKey:@"stores"]) {
                ProductStore *store = [[ProductStore alloc] init];
                store.id = [[storeObj valueForKey:@"id"] intValue];
                store.storeName = [storeObj valueForKey:@"name"];
                store.price = [[storeObj valueForKey:@"price"] floatValue];
                store.price_c = [storeObj valueForKey:@"price_c"];
                [stores addObject:store];
                [store release];
            }
            
            [product setStores:[NSArray arrayWithArray:stores]];
        }
        
        [products addObject:product];
        [product release];
    }
    
    [response setProducts:[NSArray arrayWithArray:products]];
    
    NSMutableArray *priceHints = [NSMutableArray array];
    for (id priceHintObj in [jsonResponse valueForKey:@"priceHints"]) {
        PriceHint *priceHint = [[PriceHint alloc] init];
        priceHint.u = [[priceHintObj valueForKey:@"u"] floatValue];
        priceHint.l = [[priceHintObj valueForKey:@"l"] floatValue];
        [priceHints addObject:priceHint];
        [priceHint release];
    }
    
    [response setPriceHints:[NSArray arrayWithArray:priceHints]];
    
    id priceRangeObj = [jsonResponse valueForKey:@"priceRange"];
    response.priceRangeMin = [[priceRangeObj valueForKey:@"min"] floatValue];
    response.priceRangeMax = [[priceRangeObj valueForKey:@"max"] floatValue];
    
    return [response autorelease];
}

@end
