//
//  SearchRequest.m
//  Compare Prices
//
//  Created by Jesper Forsberg on 2011-08-16.
//  Copyright 2011 Jesper Forsberg. All rights reserved.
//

#import "SearchRequest.h"
#import "Product.h"
#import "Category.h"
#import "PriceHint.h"
#import "JSONKit.h"

@implementation SearchRequest

- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
        categoryId = -1;
        query = nil;
        sort = nil;
        page = 1;
        pageSize = 10;
        minPrice = -1.f;
        maxPrice = -1.f;
        showStores = NO;
    }
    
    return self;
}

- (void)dealloc {
    [self setQuery:nil];
    [self setSort:nil];
    [super dealloc];
}

- (void)setCategoryId:(NSInteger)v
{
    categoryId = v;
}

- (void)setQuery:(NSString *)v
{
    if (v != query) {
        [query release];
        query = [v copy];
    }
}

- (void)setSort:(NSString *)v
{
    if (v != sort) {
        [sort release];
        sort = [v copy];
    }
}

- (void)setPage:(int)v
{
    page = v;
}

- (void)setPageSize:(int)v
{
    pageSize = v;
}

- (void)setMinPrice:(float)v
{
    minPrice = v;
}

- (void)setMaxPrice:(float)v
{
    maxPrice = v;
}

- (void)setShowStores:(BOOL)v
{
    showStores = v;
}

- (Category *)categoryFromJSON:(id)json
{
    Category *category = [[Category alloc] init];
    
    category.id = [[json valueForKey:@"id"] intValue];
    category.parentId = [[json valueForKey:@"parentid"] intValue];
    category.parentName = [json valueForKey:@"parentname"];
    category.name = [json valueForKey:@"name"];
    category.hitCount = [[json valueForKey:@"hitcount"] intValue];
    category.score = [[json valueForKey:@"score"] floatValue];
    
    return [category autorelease];
}

- (SearchResponse *)query:(QuenchClient *)client
{
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   [[NSNumber numberWithInt:page] stringValue], @"pg", 
                                   [[NSNumber numberWithInt:pageSize] stringValue], @"pgsize", nil];
    
    if (query) {
        [params setObject:query forKey:@"q"];
    }
    
    if (categoryId != -1) {
        [params setObject:[[NSNumber numberWithInt:categoryId] stringValue] forKey:@"id"];
    }
    
    if (sort) {
        [params setObject:sort forKey:@"s"];
    }
    
    if (minPrice != -1.f) {
        [params setObject:[[NSNumber numberWithFloat:minPrice] stringValue] forKey:@"lp"];
    }
    
    if (maxPrice != -1.f) {
        [params setObject:[[NSNumber numberWithFloat:maxPrice] stringValue] forKey:@"up"];
    }
    
    if (showStores) {
        [params setObject:@"1" forKey:@"showstores"];
    }
    
    id jsonResponse = [[client queryResource:@"/search" withParams:[NSDictionary dictionaryWithDictionary:params]] objectFromJSONData];
    
    SearchResponse *response = [[SearchResponse alloc] init];
    
    int hitCount = [[jsonResponse valueForKey:@"hitcount"] intValue];
    [response setHitCount:hitCount];
    
    int pageCount = [[jsonResponse valueForKey:@"pagecount"] intValue];
    [response setPageCount:pageCount];
    
    NSMutableArray *categories = [NSMutableArray array];
    id categoryObj = [jsonResponse valueForKey:@"categories"];
    for (id categoryKey in categoryObj) {
        [categories addObject:[self categoryFromJSON:[categoryObj valueForKey:categoryKey]]];
    }
    [response setCategories:[NSArray arrayWithArray:categories]];
    
    NSMutableArray *products = [NSMutableArray array];
    for (id productObj in [jsonResponse valueForKey:@"products"]) {
        Product *product = [[Product alloc] init];
        product.id = [[productObj valueForKey:@"id"] intValue];
        product.categoryId = [[productObj valueForKey:@"categoryid"] intValue];
        product.name = [productObj valueForKey:@"name"];
        product.info = [productObj valueForKey:@"description"];
        product.minPrice = [[productObj valueForKey:@"minprice"] floatValue];
        product.maxPrice = [[productObj valueForKey:@"maxprice"] floatValue];
        product.minPrice_c = [productObj valueForKey:@"minprice_c"];
        product.maxPrice_c = [productObj valueForKey:@"maxprice_c"];
        product.storeCount = [[productObj valueForKey:@"storecount"] intValue];
        product.imageURL = [productObj valueForKey:@"imageurl"];
        
        if (showStores) {
            NSMutableArray *stores = [NSMutableArray array];
            for (id storeObj in [productObj valueForKey:@"stores"]) {
                ProductStore *store = [[ProductStore alloc] init];
                store.storeId = [[storeObj valueForKey:@"id"] intValue];
                store.storeName = [storeObj valueForKey:@"name"];
                store.price = [[storeObj valueForKey:@"price"] floatValue];
                store.price_c = [storeObj valueForKey:@"price_c"];
                [stores addObject:store];
                [store release];
            }
            [product setStores:[NSArray arrayWithArray:stores]];
        }
        
        [products addObject:product];
        [product release];
    }
    [response setProducts:[NSArray arrayWithArray:products]];
    
    NSMutableArray *priceHints = [NSMutableArray array];
    for (id priceHintObj in [jsonResponse valueForKey:@"priceHints"]) {
        PriceHint *priceHint = [[PriceHint alloc] init];
        priceHint.u = [[priceHintObj valueForKey:@"u"] floatValue];
        priceHint.l = [[priceHintObj valueForKey:@"l"] floatValue];
        [priceHints addObject:priceHint];
        [priceHint release];
    }
    
    [response setPriceHints:priceHints];
    
    id priceRangeObj = [jsonResponse valueForKey:@"priceRange"];
    [response setPriceRangeMin:[[priceRangeObj valueForKey:@"min"] floatValue]]; 
    [response setPriceRangeMax:[[priceRangeObj valueForKey:@"max"] floatValue]];
    
    return [response autorelease];
}

@end
